<script setup>
import * as echarts from "echarts";
import "./theme/custom";
import { onMounted, ref, watchEffect, nextTick, onBeforeUnmount } from "vue";

import { throttle } from "@/assets/js/tools/throttle.js";

let myChart = null;
const chartContainer = ref(null);
const name = ref("zhongguo");
const $emit = defineEmits(["mapMounted"]);
const props = defineProps({
  roam: {
    type: [String, Boolean],
    validator: (v) => [true, false, "scale", "move"].includes(v),
    default: true,
  },
});
let geoJson = {};
async function getGeo() {
  geoJson = (await import(`../../../public/mapJson/${name.value}.json`))
    .default;
  echarts.registerMap(name.value, geoJson);
}

function getOption(list = []) {
  let mapName = name.value;

  let data = list;

  let areaList = geoJson.features || [];
  let data0 = [];
  areaList.forEach((v) => {
    data0.push({
      name: v.properties.name,
      value: Math.floor(Math.random() * 2) * 100,
    });
  });

  let convertData = function (data = []) {
    let res = [];
    for (let i = 0; i < data.length; i++) {
      let item = data[i];
      res.push({
        // name: item.name,
        color: item.color,
        value: item.points.concat(data[i].value),
        label: item.label ? item.label : "",
        data: item,
      });
    }
    return res;
  };
  return {
    tooltip: {
      show: true,
      formatter(params) {
        // console.log(params);
        return params.data.label;
      },
    },
    geo: {
      show: true,
      map: mapName,
      roam: props.roam,
      itemStyle: {
        borderColor: "rgba(147, 235, 248, 1)",
        borderWidth: 1,
        areaColor: {
          type: "radial",
          x: 0.5,
          y: 0.5,
          r: 0.8,
          colorStops: [
            {
              offset: 0,
              color: "rgba(147, 235, 248, 0)", // 0% 处的颜色
            },
            {
              offset: 1,
              color: "rgba(147, 235, 248, .2)", // 100% 处的颜色
            },
          ],
          globalCoord: false, // 缺省为 false
        },
        shadowColor: "rgba(128, 217, 248, 1)",
        shadowOffsetX: -2,
        shadowOffsetY: 2,
        shadowBlur: 10,
      },
      emphasis: {
        itemStyle: {
          areaColor: "#1ecee5",
          borderWidth: 0,
        },
        label: {
          show: false,
        },
      },
    },
    series: [
      // 图片底色
      {
        name: "",
        type: "scatter",
        coordinateSystem: "geo",
        symbol: "circle",
        symbolSize: [24, 24],
        label: {
          formatter: "{b}",
          position: "inside",
          show: true,
          color: "#FFFFFF",
          fontWeight: "bold",
        },
        itemStyle: {
          color(params) {
            return params.data.color;
          },
        },
        data: convertData(data),
        z: 3,
      },
      {
        //图片
        name: "图片",
        type: "scatter",
        coordinateSystem: "geo",
        symbol: function (value, params) {
          return "circle";
        },
        symbolSize: [24, 24],
        label: {
          formatter: "{b}",
          position: "inside",
          show: true,
          color: "#FFFFFF",
          fontWeight: "bold",
        },
        itemStyle: {
          color(params) {
            return params.data.color;
          },
        },
        data: convertData(data),
        showEffectOn: "render",
        rippleEffect: {
          brushType: "stroke",
        },
        emphasis: {
          scale: true
        },
        // zlevel: 3
        z: 4,
      },
      {
        name: "波纹",
        type: "effectScatter",
        coordinateSystem: "geo",
        rippleEffect: {
          scale: 3,
        },
        symbolSize: [18, 18],
        data: convertData(data),
        itemStyle: {
          color(params) {
            return params.data.color;
          },
        },
        tooltip: {
          show: true,
        },
        // zlevel: 2
        z: 2,
      },
      {
        type: "map",
        map: mapName,
        geoIndex: 0,
        aspectScale: 0.75, //长宽比
        showLegendSymbol: false, // 存在legend时显示
        label: {
          show: true,
        },
        emphasis: {
          show: false,
          textStyle: {
            color: "#FF0990",
          },
        },
        itemStyle: {
          areaColor: "#FF9900",
        },
        roam: true,
        animation: false,
        data: data0,
        // data: [],
        tooltip: {
          show: false,
        },
      },
    ],
  };
}
function handleClick(params) {
  if (params.seriesIndex == 0) {
    let data = params.data;
    // $emit("handleClick", data.data);
  }
}
function initChart() {
  if (chartContainer.value) {
    myChart = echarts.init(chartContainer.value, "custom");

    let option = getOption();
    console.log("time 2", Date.now(), option, "=============");

    myChart.setOption(option);

    // 监听窗口变化重新渲染图表
    window.addEventListener("resize", () => {
      myChart.resize();
    });

    myChart.off("click"); //防止chart点击触发多次
    myChart.on("click", (params) => {
      handleClick(params);
    });
    myChart.off("georoam");
    myChart.on(
      "georoam",
      throttle(() => {
        let option = myChart.getOption();
        myChart.setOption(option);
      }, 100)
    );
  }
}

function updateChart(data) {
  let option = getOption(data);

  myChart?.setOption(option);
}

defineExpose({
  updateChart,
});

onMounted(async () => {
  await getGeo();
  initChart();
  $emit("mapMounted");
});
</script>

<template>
  <div class="chart-wraper">
    <div class="chart" ref="chartContainer"></div>
    <slot name="tools"></slot>
  </div>
</template>

<style scoped lang="less">
.chart-wraper {
  height: 100%;
  position: relative;
  .chart {
    position: absolute;
    left: 0;
    top: 0;
    right: 0;
    bottom: 0;
  }
}
</style>
